@@ -13,6 +13,9 @@ class ChannelError(amfast.AmFastError):
 class NotConnectedError(ChannelError):
     pass
 
+class TimeOutError(ChannelError):
+    pass
+
 class SecurityError(ChannelError):
     pass
 
@@ -46,18 +49,25 @@ class Connection(object):
      * active - boolean, True if Connection is in use
     """
 
+    NOTIFY_KEY = '_notify'
+
     def __init__(self, flex_client_id, channel):
         self.authenticated = False
         self.last_active = int(time.time())
         self.active = True
-        self.connected = False
 
+        self._lock = threading.RLock()
         self._flex_client_id = flex_client_id # Unique for each Flex client.
         self._channel = channel
         self._messages = []
         self._subscriptions = {}
         self._session_attrs = {}
 
+        # Version 0.4 will use Beaker to store session data.
+        # all 'new' attributes should be keyed to help compatibility
+        # with upgrade to version 0.4.
+        self.setSessionAttr(self.NOTIFY_KEY, False)
+
     # channel should be read only
     def _getChannel(self):
         return self._channel
@@ -84,12 +94,7 @@ class Connection(object):
         ==========
          * attr - string, name of attribute to retrieve.
         """
-        lock = threading.RLock()
-        lock.acquire()
-        try:
-            return self._session_attrs[attr]
-        finally:
-            lock.release()
+        return self._session_attrs[attr]
 
     def delSessionAttr(self, attr):
         """Remove a session attribute.
@@ -98,14 +103,12 @@ class Connection(object):
         ==========
          * attr - string, name of attribute to remove.
         """
-
-        lock = threading.RLock()
-        lock.acquire()
+        self._lock.acquire()
         try:
             if attr in self._session_attrs:
                 del self._session_attrs[attr]
         finally:
-            lock.release()
+            self._lock.release()
 
     def setSessionAttr(self, attr, val):
         """Set a session attribute.
@@ -115,13 +118,7 @@ class Connection(object):
          * attr - string, name of attribute to set.
          * val - object, value to set.
         """
-
-        lock = threading.RLock()
-        lock.acquire()
-        try:
-            self._session_attrs[attr] = val
-        finally:
-            lock.release()
+        self._session_attrs[attr] = val
 
     def touch(self):
          """Updates last_active to the current time."""
@@ -134,17 +131,17 @@ class Connection(object):
 
     def getSubscriptions(self):
         """Returns a list of subscription objects."""
+
         subscriptions = []
 
         # Is there a way to make a thread-safe
         # generator for this? ...don't think so.
-        lock = threading.RLock()
-        lock.acquire()
+        self._lock.acquire()
         try:
             for client_subscriptions in self._subscriptions.values():
                 subscriptions.extend(client_subscriptions.values())
         finally:
-            lock.release()
+            self._lock.release()
 
         return subscriptions
 
@@ -158,8 +155,7 @@ class Connection(object):
         """
         subscription = Subscription(self, client_id, topic)
 
-        lock = threading.RLock()
-        lock.acquire()
+        self._lock.acquire()
         try:
             client_subscriptions = self._subscriptions.get(client_id, None)
             if client_subscriptions is None:
@@ -167,7 +163,7 @@ class Connection(object):
                 self._subscriptions[client_id] = client_subscriptions
             client_subscriptions[topic] = subscription
         finally:
-            lock.release()
+            self._lock.release()
 
         return subscription
 
@@ -180,15 +176,17 @@ class Connection(object):
          * topic - string, the topic to un-subscribe from.
         """
 
-        lock = threading.RLock()
-        lock.acquire()
+        self._lock.acquire()
         try:
             client_subscriptions = self._subscriptions.get(client_id, None)
             if client_subscriptions is not None:
                 if topic in client_subscriptions:
                     del client_subscriptions[topic]
+
+                if len(client_subscriptions) < 1:
+                    del self._subscriptions[client_id]
         finally:
-            lock.release()
+            self._lock.release()
 
     def poll(self):
         """Returns all current messages and empties que."""
@@ -199,12 +197,7 @@ class Connection(object):
 
     def popMessage(self):
         """Remove and return a single message from the qeue."""
-        lock = threading.RLock()
-        lock.acquire()
-        try:
-            return self._messages.pop(0)
-        finally:
-            lock.release()
+        return self._messages.pop()
 
     def hasMessages(self):
         """Returns True if messages are present."""
@@ -217,12 +210,12 @@ class Connection(object):
         ==========
          * msg, AbstractMessage, message to publish.
         """
-        lock = threading.RLock()
-        lock.acquire()
-        try:
-            self._messages.append(msg)
-        finally:
-            lock.release()
+        self._messages.append(msg)
+
+        notify_func = self.getSessionAttr(self.NOTIFY_KEY)
+        if notify_func is not False:
+            # Notify that a message has been published
+            notify_func()
 
     def clean(self, current_time=None):
         """Remove all expired messages."""
@@ -230,8 +223,7 @@ class Connection(object):
             current_time = int(time.time())
 
         tmp = []
-        lock = threading.RLock()
-        lock.acquire()
+        self._lock.acquire()
         try:
              for msg in self._messages:
                  if msg.isExpired(current_time):
@@ -239,32 +231,7 @@ class Connection(object):
                  tmp.append(msg)
              self._messages = tmp
         finally:
-            lock.release()
-
-class StreamingConnection(Connection):
-    """Publish messages directly to stream, instead of to the qeue.
-
-     attributes
-     ===========
-     * connected - boolean, True if Connection is connected and streaming
-     * heart_interval - int, Number of seconds between heart beat responses.
-     * channel_publish - bool, True if self.publish should be called.
-     """
-    def __init__(self, flex_client_id, channel,
-        channel_publish=True, connected=False,
-        heart_interval=30):
-
-        Connection.__init__(self, flex_client_id, channel)
-        self.connected = connected
-        self.heart_interval = heart_interval
-        self.channel_publish = channel_publish
-
-    def publish(self, msg):
-        self.touch() # I touch myself, but only when no one is looking to make sure my connection stays alive
-        if self.channel_publish is True and self.connected is True:
-            self.channel.publish(self, msg)
-        else:
-            Connection.publish(self, msg)
+            self._lock.release()
 
 class Channel(object):
     """An individual channel that can send/receive messages.
@@ -276,22 +243,21 @@ class Channel(object):
      * max_connections - int, When the number of connections exceeds this number,
          an exception is raised when new clients attempt to connect. Set to -1
          for no limit.
-     * connection_class - Connection, The class to use for new connections.
      * timeout - int, If a Connection's last_active value is < current time - timeout,
          the Connection will be disconnected.
 
     """
 
-    def __init__(self, name, max_connections=-1, endpoint=None,
-        timeout=1800, connection_class=Connection):
+    def __init__(self, name, max_connections=-1, endpoint=None, timeout=1200):
         self.name = name
         self.max_connections = max_connections
-        self.connection_class = connection_class
+        self.connection_count = 0
         self.timeout = timeout
         if endpoint is None:
             endpoint = AmfEndpoint()
         self.endpoint = endpoint
-        
+       
+        self._lock = threading.RLock() 
         self._channel_set = None
 
     def _getChannelSet(self):
@@ -335,11 +301,24 @@ class Channel(object):
 
         Returns Connection
         """
-        if self.max_connections > -1 and len(self._connections) >= self.max_connections:
+        if self.max_connections > -1 and self.connection_count >= self.max_connections:
             raise ChannelError("Channel '%s' is not accepting connections." % self.name)
 
-        connection = self.connection_class(flex_client_id, self)
-        self._channel_set.addConnection(self.connection_class(flex_client_id, self))
+        try:
+            self._channel_set.getConnection(flex_client_id)
+        except NotConnectedError:
+            pass
+        else:
+            raise ChannelError('Client ID is already connected.')
+ 
+        self._lock.acquire()
+        try:
+            self.connection_count += 1
+        finally:
+            self._lock.release()
+
+        connection = Connection(flex_client_id, self)
+        self._channel_set.addConnection(connection)
 
         return connection
 
@@ -350,6 +329,12 @@ class Channel(object):
         ==========
          * flex_client_id - string, Flex client id.
         """
+        self._lock.acquire()
+        try:
+            self.connection_count -= 1
+        finally:
+            self._lock.release()
+
         self.channel_set.disconnect(connection)
 
 class HttpChannel(Channel):
@@ -360,27 +345,17 @@ class HttpChannel(Channel):
      * wait_interval - int, Number of seconds to wait before sending response to client
          when a polling request is received. Set to -1 to configure channel as a
          long-polling channel.
-     * check_interval - int or float, Number of seconds to wait between message
-         qeue checks when checking for new messages. Internal polling interval.
     """
 
     # Content type for amf messages
     CONTENT_TYPE = 'application/x-amf'
 
     def __init__(self, name, max_connections=-1, endpoint=None,
-        timeout=1800, connection_class=Connection, wait_interval=0,
-        check_interval=0.1):
+        timeout=1800, wait_interval=0):
 
-        Channel.__init__(self, name, max_connections, endpoint,
-            timeout, connection_class)
+        Channel.__init__(self, name, max_connections, endpoint, timeout)
 
-        self._wait_interval = wait_interval
-        self.check_interval = check_interval
-
-    # wait_interval should be read-only
-    def _getWaitInterval(self):
-        return self._wait_interval
-    wait_interval = property(_getWaitInterval)
+        self.wait_interval = wait_interval
 
     def waitForMessage(self, packet, message, connection):
         """Waits for a new message.
@@ -388,20 +363,14 @@ class HttpChannel(Channel):
         This is blocking, and should only be used
         for Channels where each connection is a thread.
         """
-        total = 0
-        while True:
-            if connection.active is False:
-                return
+        event = threading.Event()
+        connection.setSessionAttr(connection.NOTIFY_KEY, event.set)
+        if (self.wait_interval > -1):
+            event.wait(self.wait_interval)
 
-            if connection.hasMessages():
-                return
-
-            if self.wait_interval > 0 and total > self.wait_interval:
-                # Max wait interval reached.
-                return
-
-            time.sleep(self.check_interval)
-            total += self.check_interval
+        # Event.set has been called,
+        # or timeout has been reached
+        connection.setSessionAttr(connection.NOTIFY_KEY, False)
 
 class ChannelSet(object):
     """A collection of Channels.
@@ -429,13 +398,16 @@ class ChannelSet(object):
             message_agent = MessageAgent()
         self.message_agent = message_agent
 
+        self._lock = threading.RLock()
         self._channels = {}
         self._connections = {}
         self.clean_freq = clean_freq
         self._last_cleaned = int(time.time())
 
     def __iter__(self):
-        return self._channels.itervalues()
+        # To make thread safe
+        channel_vals = self._channels.values()
+        return channel_vals.__iter__
 
     def checkCredentials(self, user, password):
         """Determines if credentials are valid.
@@ -459,12 +431,7 @@ class ChannelSet(object):
 
     def addConnection(self, connection):
         """Add a connection to the ChannelSet."""
-        lock = threading.RLock()
-        lock.acquire()
-        try:
-            self._connections[connection.flex_client_id] = connection
-        finally:
-            lock.release()
+        self._connections[connection.flex_client_id] = connection
 
     def getConnection(self, flex_client_id):
         """Retrieve an existing connection.
@@ -475,8 +442,7 @@ class ChannelSet(object):
         """
         current_time = int(time.time())
 
-        lock = threading.RLock()
-        lock.acquire()
+        self._lock.acquire()
         try:
             connection = self._connections.get(flex_client_id, None)
 
@@ -485,12 +451,14 @@ class ChannelSet(object):
 
             cutoff_time = current_time - connection.channel.timeout
             if connection.last_active < cutoff_time:
-                connection.disconnect()
-                raise NotConnectedError('Client is not connected.')
+                raise TimeOutError('Connection is timed out.')
             
             connection.touch()
+        except TimeOutError:
+            connection.disconnect()
+            raise NotConnectedError('Client is not connected.')
         finally:
-            lock.release()
+            self._lock.release()
 
         # Clean up dead connections and subscriptions
         # TODO: there's got to be a better way to do this
@@ -506,19 +474,19 @@ class ChannelSet(object):
         ==========
          * flex_client_id - string, id of client to get connection for.
         """
-        lock = threading.RLock()
-        lock.acquire()
-        try:
-           # Delete any subscriptions
-           subscriptions = connection.getSubscriptions()
-           for subscription in subscriptions:
-               self.message_agent.unsubscribe(subscription.connection,
-                   subscription.client_id, subscription.topic)
+        subscriptions = connection.getSubscriptions()
 
-           if connection.flex_client_id in self._connections:
-               del self._connections[connection.flex_client_id]
+        # Delete any subscriptions
+        for subscription in subscriptions:
+           self.message_agent.unsubscribe(subscription.connection,
+               subscription.client_id, subscription.topic, _disconnect=True)
+
+        self._lock.acquire()
+        try:
+            if connection.flex_client_id in self._connections:
+                del self._connections[connection.flex_client_id]
         finally:
-            lock.release()
+            self._lock.release()
 
     def getFlexConnection(self, packet, msg):
         """Returns a Connection object for a Flex message.
@@ -577,13 +545,12 @@ class ChannelSet(object):
         ==========
          * channel - Channel, the channel to add.
         """
-        lock = threading.RLock()
-        lock.acquire()
+        self._lock.acquire()
         try:
             self._channels[channel.name] = channel
             channel._channel_set = self
         finally:
-            lock.release()
+            self._lock.release()
 
     def unMapChannel(self, channel):
         """Removes a Channel to the ChannelSet
@@ -592,14 +559,13 @@ class ChannelSet(object):
         ==========
          * channel - Channel, the channel to remove.
         """
-        lock = threading.RLock()
-        lock.acquire()
+        self._lock.acquire()
         try:
             if channel.name in self._channels:
                 channel._channel_set = None
                 del self._channels[channel.name]
         finally:
-            lock.release()
+            self._lock.release()
 
     def getChannel(self, name):
         """Retrieves a Channel from the ChannelSet
@@ -608,14 +574,7 @@ class ChannelSet(object):
         ==========
          * name - string, the name of the Channel to retrieve.
         """
-
-        lock = threading.RLock()
-        lock.acquire()
-        try:
-            channel = self._channels[name]
-        finally:
-            lock.release()
-        return channel
+        return self._channels[name]
 
     def clean(self):
         """Remove inactive Connections and Messages.
@@ -636,6 +595,6 @@ class ChannelSet(object):
         for connection in self._connections.values():
             cutoff = current_time - connection.channel.timeout
             if connection.last_active < cutoff:
-                self.disconnect(connection.flex_client_id)
+                self.disconnect(connection)
             else:
                 connection.clean(current_time)
