@@ -15,22 +15,20 @@ def amfhook():
 cherrypy.tools.amfhook = cherrypy.Tool('before_request_body', amfhook, priority=0)
 
 class CherryPyChannel(HttpChannel):
-    """An AMF RPC channel that can be used with CherryPy HTTP framework.
+    """An AMF messaging channel that can be used with CherryPy HTTP framework.
+
+    Instantiate a CherryPyChannel object and
+    mount the processMsg method to the URL where
+    AMF messaging should be available from. 
+
+    CherryPyChannel does not support HTTP streaming.
+    Use WsgiChannel to achieve HTTP streaming.
 
-    CherryPyChannel is deprecated.
-    Please use WsgiChannel instead.
     A WsgiChannel instance can be mounted to a URL
     within CherryPy with the command cherrypy.tree.graft.
 
     """
 
-    def __init__(self, *args, **kwargs):
-        print "CherryPyChannel is deprecated." \
-            "Use WsgiChannel instead." \
-            "A WsgiChannel instance can be mounted to a URL" \
-            "within CherryPy with the command cherrypy.tree.graft."
-        HttpChannel.__init__(self, *args, **kwargs)
-
     @cherrypy.expose
     @cherrypy.tools.amfhook()
     def processMsg(self):
@@ -43,7 +41,3 @@ class CherryPyChannel(HttpChannel):
         response = self.invoke(self.decode(raw_request))
         cherrypy.response.headers['Content-Type'] = self.CONTENT_TYPE
         return self.encode(response)
-
-    def waitForMessage(self, packet, message, connection):
-        cherrypy.request.timeout = 1000000
-        Channel.waitForMessage(self, packet, message, connection)
